==> xl-react-components-0.16.62.tgz: package/app/xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings.js <==
// eslint-disable-line max-lines
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import {getModelValue, isDisabled, resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextPropsState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {sortStrings} from '../../utils/sorting-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipSetOfStringsComponent} from './xl-react-dip-set-of-strings-component';
import * as service from './xl-react-dip-set-of-strings-service';
import {getUpdatedCellState} from './set-of-strings-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipSetOfStrings extends Component {
    static propTypes = {
        model: PropTypes.array,
        showNewKeyValueRow: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);

        this.state = {
            newKey: '',
            searchWord: '',
            showNewKeyValueRow: false
        };
    }

    componentWillMount() {
        const model = getModelValue(this.props, 'model', []);
        computeInitialState(this, this.props, getValidators(this.props), model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
        if (this.state.pristine) {
            const modelKeys = service.getModelKeys(this.state.model, true);
            this.setState({modelKeys});
        }
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    /* eslint-disable complexity */
    handleKeyBlur = (changedKey, oldKey) => {
        const {model} = this.state;
        if (!R.contains(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const keyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({
                    emptyKey: true,
                    model: R.clone(model)
                });
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                const newState = {
                    duplicateKey: trimmedChangedKey,
                    model: R.clone(model)
                };
                this.setState(newState);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedInputKeyFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState({
                    ...this.state,
                    model: updatedModel
                }, focusToInputField);
            }
        };

        if (!keyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedInputKeyFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };
    /* eslint-enable complexity */

    handleRemoveKey = (key) => {
        const model = R.without(key, this.state.model);
        this.applyNewState({
            ...this.state,
            model
        }, this.focusToInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        const {cellState, model} = this.state;
        this.setState({
            cellState: getUpdatedCellState(model, cellState, {sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: 'key',
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedInputKeyFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state);
        const applyModelIfPossible = () => {
            const {newKey, newValueFocused} = this.state;
            const trimmedNewKey = R.trim(newKey);

            if (!newValueFocused && trimmedNewKey) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => {
        setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);
    };

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    delayedInputKeyFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey, model} = this.state;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                newKey: '',
                showNewKeyValueRow: false
            });
            const focusToField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState({
                model: [
                    ...model,
                    newKey
                ]
            }, focusToField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusToInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedInputKeyFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedInputKeyFocus();
            });
        },
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    applyNewState = ({duplicateKey, hasNewKeyError, model}, callback) => {
        const customCallbacks = callback ? [callback] : [];
        this.setState({
            duplicateKey,
            emptyKey: undefined,
            hasNewKeyError
        });
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {displayMode, metadata, onValidityChange} = this.props;
        const {
            message,
            cellState,
            duplicateKey,
            emptyKey,
            model,
            newKey,
            pristine,
            showNewKeyValueRow,
            searchWord,
            valid,
            sorting
        } = this.state;

        const {
            autoHide, canAddNewItem, description, label, password, required
        } = metadata;

        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipSetOfStringsComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            canAddNewItem,
            cellState,
            description,
            disabled,
            duplicateKey,
            emptyKey,
            encrypted: password,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model: sortStrings(model, sorting ? sorting.sortDirection : 'ASC'),
            newKey,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}

// eslint-enable-line max-lines

==> xl-react-components-0.18.103.tgz: package/app/xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings.js <==
// eslint-disable-line max-lines
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import {isDisabled, resetPristinity} from '../../utils/components-utils';
import {
    computeInitialState,
    computeNextPropsState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {sortStrings} from '../../utils/sorting-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipSetOfStringsComponent} from './xl-react-dip-set-of-strings-component';
import * as service from './xl-react-dip-set-of-strings-service';
import {getUpdatedCellState} from './set-of-strings-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipSetOfStrings extends Component {
    static propTypes = {
        model: PropTypes.array,
        showNewKeyValueRow: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);

        this.state = {
            newKey: '',
            searchWord: '',
            showNewKeyValueRow: false
        };
    }

    componentWillMount() {
        computeInitialState(this, this.props, getValidators(this.props), this.props.model);
    }

    componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
        if (this.state.pristine) {
            const modelKeys = service.getModelKeys(this.state.model, true);
            this.setState({modelKeys});
        }
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    /* eslint-disable complexity */
    handleKeyBlur = (changedKey, oldKey) => {
        const {model} = this.state;
        if (!R.contains(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const keyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({
                    emptyKey: true,
                    model: R.clone(model)
                });
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                const newState = {
                    duplicateKey: trimmedChangedKey,
                    model: R.clone(model)
                };
                this.setState(newState);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedInputKeyFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState({
                    ...this.state,
                    model: updatedModel
                }, focusToInputField);
            }
        };

        if (!keyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedInputKeyFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };
    /* eslint-enable complexity */

    handleRemoveKey = (key) => {
        const model = R.difference(this.props.model, [key]);
        this.applyNewState({
            ...this.state,
            model
        }, this.focusToInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        const {cellState, model} = this.state;
        this.setState({
            cellState: getUpdatedCellState(model, cellState, {sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: 'key',
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedInputKeyFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state);
        const applyModelIfPossible = () => {
            const {newKey, newValueFocused} = this.state;
            const trimmedNewKey = R.trim(newKey);

            if (!newValueFocused && trimmedNewKey) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => {
        setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);
    };

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    delayedInputKeyFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey, model} = this.state;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                newKey: '',
                showNewKeyValueRow: false
            });
            const focusToField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState({
                model: [
                    ...model,
                    newKey
                ]
            }, focusToField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusToInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedInputKeyFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedInputKeyFocus();
            });
        },
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    applyNewState = ({duplicateKey, hasNewKeyError, model}, callback) => {
        const customCallbacks = callback ? [callback] : [];
        this.setState({
            duplicateKey,
            emptyKey: undefined,
            hasNewKeyError
        });
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {displayMode, metadata, onValidityChange} = this.props;
        const {
            message,
            cellState,
            duplicateKey,
            emptyKey,
            model,
            newKey,
            pristine,
            showNewKeyValueRow,
            searchWord,
            valid,
            sorting
        } = this.state;

        const {
            autoHide, canAddNewItem, description, label, password, required
        } = metadata;

        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipSetOfStringsComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            canAddNewItem,
            cellState,
            description,
            disabled,
            duplicateKey,
            emptyKey,
            encrypted: password,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model: sortStrings(model, sorting ? sorting.sortDirection : 'ASC'),
            newKey,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}

// eslint-enable-line max-lines
